/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: ADF.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include "ADF.h"
#include <math.h>
#include <qpixmap.h>

#include "Embed.h"

ADF::ADF(QWidget *parent, const char *name, WFlags flags)
	:CompassInstrument(parent, name, flags)
{
	;
}

const QPixmap &
ADF::overlayPixmap()
{
	static const QPixmap *p;

	if (!p)
		p = &embed_QPixmap("images/ADFOverlay.png");
	return *p;
}

void
ADF::render()
{
	CompassInstrument::render();
	int b = nav_radio ? nav_radio->relativeBearing() : 0;
	drawBearingArrow(b);
}

void
ADF::drawBearingArrow(int b)
{
	QPainter p(&pm);
	p.setPen(QPen(Qt::yellow, 2,
			Qt::SolidLine,
			Qt::SquareCap,
			Qt::BevelJoin));
	const int size = pm.height() / 16 * 6;
	QPoint center(pm.width() / 2, pm.height() / 2);
	double angle = -(b / 180. * M_PI) + M_PI/2;
	QPoint head(center.x() + cos(angle) * size,
		    center.y() + sin(angle) * size);
	QPointArray arrow(5);
	arrow[0] = QPoint(2*center.x()-head.x(), 2*center.y()-head.y());
	arrow[1] = head;
	const double headAng = 160. / 180. * M_PI;
	double a2 = angle + headAng;
	arrow[2] = QPoint(head.x() + cos(a2) * size/4,
			  head.y() + sin(a2) * size/4);
	arrow[3] = head;
	a2 = angle - headAng;
	arrow[4] = QPoint(head.x() + cos(a2) * size/4,
			  head.y() + sin(a2) * size/4);

	p.drawPolyline(arrow);
}

// XXX refac
void
ADF::mouseSetHeading(const QPoint &p, bool initial = false)
{
	if (!is_slaved) {
		CompassInstrument::mouseSetHeading(p, initial);
	}
}
