/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: Airplane.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include <math.h>

#include "Airplane.h"

#include "DG.h"
#include "CDI.h"
#include "ADF.h"
#include "RMI.h"

Airplane::Airplane(QCanvas *canvas)
{
	canv_ap = new SimCanvasAirplane(canvas);
	canv_ap->show();
	connect(canv_ap, SIGNAL(dragged()), this, SIGNAL(changed()));

	for (int i = 0; i < NNAV; ++i) {
		navs[i] = new NAV();
		navs[i]->setAirplane(this);
	}

	instruments[id_DG] = new DG(0, "DG");
	instruments[id_DG]->setNAV(navs[0]);
	connect(instruments[id_DG], SIGNAL(headingChanged(int)),
		this, SLOT(updateHeading()));

	instruments[id_VOR1] = new CDI(0, "VOR 1");
	instruments[id_VOR1]->setNAV(navs[0]);
	instruments[id_VOR2] = new CDI(0, "VOR 2");
	instruments[id_VOR2]->setNAV(navs[1]);
	instruments[id_ADF] = new ADF(0, "ADF");
	instruments[id_ADF]->setNAV(navs[2]);
	instruments[id_RMI] = new RMI(0, "RMI");
	instruments[id_RMI]->setNAV(navs[2]);
	instruments[id_RMI]->setNAV(navs[0]);
}

void
Airplane::updateHeading()
{
	int h = instruments[id_DG]->heading();
	if (h != heading()) {
		canv_ap->setAngle(h / 180. * M_PI);
		emit changed();
	}
}

int
Airplane::heading() const
{
	int h = canv_ap->angle() / M_PI * 180.;
	return (h + 720) % 360;
}

void
Airplane::move(int x, int y)
{
	canv_ap->move(x, y);
	emit changed();
}
