/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: CDI.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include "CDI.h"
#include <math.h>
#include <qpixmap.h>
#include <qnamespace.h>

#include "Embed.h"

QPixmap *CDI::flagPixmaps[3];
bool CDI::flagPixmapsDrawn;

CDI::CDI(QWidget *parent, const char *name, WFlags flagPixmaps)
	:CompassInstrument(parent, name, flagPixmaps)
{
	if (!flagPixmapsDrawn) {
		const int w = (int)(overlayPixmap().width() / 8) | 1, h = w / 2;

		CDI::flagPixmaps[NAV::Off] = new QPixmap(w, h);
		CDI::flagPixmaps[NAV::Off]->fill(Qt::white);
		QPainter poff(CDI::flagPixmaps[NAV::Off]);
		poff.setPen(QPen(Qt::red, w / 6));
		for (int i = -w * 2; i < 2 * w; i += w / 3)
			poff.drawLine(i, h*2, i + 2*w, -h*2);

		CDI::flagPixmaps[NAV::To] = new QPixmap(w, h);
		CDI::flagPixmaps[NAV::To]->fill(Qt::black);
		QPointArray a(3);
		a[0] = QPoint(w / 2, 0);
		a[1] = QPoint(0, h - 1);
		a[2] = QPoint(w - 1, h - 1);
		QPainter pto(CDI::flagPixmaps[NAV::To]);
		pto.setBrush(Qt::white);
		pto.drawPolygon(a);

		CDI::flagPixmaps[NAV::From] = new QPixmap(w, h);
		CDI::flagPixmaps[NAV::From]->fill(Qt::black);
		a[0] = QPoint(w / 2, h - 1);
		a[1] = QPoint(w - 1, 0);
		a[2] = QPoint(0, 0);
		QPainter pfrom(CDI::flagPixmaps[NAV::From]);
		pfrom.setBrush(Qt::white);
		pfrom.drawPolygon(a);

		flagPixmapsDrawn = TRUE;
	}
	connect(this, SIGNAL(headingChanged(int)),
		this, SLOT(render()));
}

const QPixmap &
CDI::overlayPixmap()
{
	static const QPixmap *p;

	if (!p)
		p = &embed_QPixmap("images/CDIOverlay.png");

	return *p;
}

void
CDI::render()
{
	CompassInstrument::render();
	double defl;
	int flag;

	if (nav_radio) {
		flag = nav_radio->CDIInfo(defl, hdg);
	} else {
		flag = NAV::Off;
		defl = 0.;
	}

	QPainter p(&pm);
	p.setPen(QPen(Qt::yellow, 2, Qt::SolidLine,
			Qt::SquareCap, Qt::MiterJoin));
	const int size = pm.height();
	QPoint center(pm.width() / 2, size / 5);
	double angle = (defl / 10.) * 35. / 180. * M_PI;
	QPoint tail(center.x() + sin(angle) * size * 0.55,
		    center.y() + cos(angle) * size * 0.55);
	p.drawPixmap(pm.width() / 5 * 3, pm.height() / 10 * 4,
			*flagPixmaps[flag]);
	p.drawLine(center, tail);
}
