/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: CanvasRPolygon.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include <math.h>
#include <qpainter.h>
#include "CanvasRPolygon.h"

const int CanvasRPolygon::RTTI =    814178733;

CanvasRPolygon::CanvasRPolygon(QCanvas *canvas, double a = 0)
	: QCanvasPolygon(canvas)
{
	setPen(NoPen);
	setAngle(a);
	connect(this, SIGNAL(dragged()), this, SLOT(render()));
}

void
CanvasRPolygon::setBasePoints(QPointArray const &a)
{
	basepoints = a;
	render();
}

void
CanvasRPolygon::render()
{
	double a = angle();
	double c = cos(a);
	double s = sin(a);

	QPointArray pa(basepoints.size());
	int n = 0;
	for (int i = 0; i < (int) basepoints.size(); i++) {
		int x = basepoints[i].x(), y = basepoints[i].y();
		pa[n++] = QPoint(c * x - s * y,
				s * x + c * y);
	}
	setPoints(pa);
	canvas()->update();
}

void
CanvasRPolygon::setAngle(double a)
{
	DragCanvasItemI::setAngle(a);
	render();
}
