/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: CompassInstrument.h,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#ifndef __COMPASSINSTRUMENT_H
#define __COMPASSINSTRUMENT_H

#include <qframe.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qimage.h>
#include "NAV.h"

class NAV;

class CompassInstrument : public QFrame
{
	Q_OBJECT
public:
	CompassInstrument(QWidget *parent = 0, const char *name = 0, WFlags flags = 0);
	virtual void setNAV(NAV *n);

	int	heading() const { return hdg; }
public slots:
	void	setHeading(int);
	void	updateHeading();

signals:
	void	headingChanged(int);

protected:
	virtual void mouseMoveEvent(QMouseEvent *);
	virtual void mousePressEvent(QMouseEvent *);
	virtual void drawContents(QPainter *);
	virtual void resizeEvent(QResizeEvent *);

public slots:
	void		dirty();

protected:
	virtual void	render();
	virtual void	mouseSetHeading(const QPoint &, bool initial = false);
	int		hdg;
	NAV		*nav_radio;

	uint		dirtyf:1;		// pixmap needs to be updated
	uint		is_slaved:1;
	uint		is_fixed:1;
	QPixmap		pm;
	QPixmap		facePixmap;
	QImage		rotatedCompass;
	int		rotatedCompass_hdg;

	virtual const QPixmap &overlayPixmap() = 0;
};

#endif // __COMPASSINSTRUMENT_H
