/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: LineIterator.h,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#ifndef __LINE_ITERATOR_H
#define __LINE_ITERATOR_H

//
// Bresenham's Line Rasterizing Algorithm.
//
// Create an instance by specifying endpoints, then iterate over the points
// along the line.
//
// LineIterator l(1, 2, 17, 31);
// while (l.next()) {
//         operateOn(l.x, l.y);
// }
//
class LineIterator
{
public:
	int x, y;

	LineIterator(int x1, int y1, int x2, int y2)
	 : x(x1), y(y1), x2(x2), y2(y2) {
		wx = x2 - x;
		wy = y2 - y;

		if (wx < 0) {
			wx = -wx;
			dx = -1;
		} else
			dx = 1;

		if (wy < 0) {
			wy = -wy;
			dy = -1;
		} else
			dy = 1;

		state = LIS_INITIAL;
	}

	bool next() {
		switch (state) {
		case LIS_FINI:
			return false;

		case LIS_INITIAL:
			if (wx > wy) {
				e = wx / 2;
				state = LIS_XMAJOR;
			} else {
				e = wy / 2;
				state = LIS_YMAJOR;
			}
			return true;

		case LIS_XMAJOR:
			if (x != x2) {
				x += dx;
				e += wy;
				if (e >= wx) {
					e -= wx;
					y += dy;
				}
			} else {
				y = y2;
				state = LIS_FINI;
				return false;
			}
			return true;

		case LIS_YMAJOR:
			if (y != y2) {
				y += dy;
				e += wx;
				if (e >= wy) {
					e -= wy;
					x += dx;
				}
			} else {
				x = x2;
				state = LIS_FINI;
				return false;
			}
			return true;
		}
		// NOTREACHED
		return false;
	}

private:
	enum { LIS_INITIAL, LIS_XMAJOR, LIS_YMAJOR, LIS_FINI } state;
	const int x2, y2;
	int wx, wy, dx, dy;
	int e;

};

#endif // __LINE_ITERATOR_H
