/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: NAV.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include "NAV.h"

NAV::NAV()
	:ap(0), nav(0)
{
	;
}

bool
NAV::isTuned() const
{
	return ap && nav && nav->isVisible();
}

void
NAV::setAirplane(Airplane *a)
{
	if (ap)
		disconnect(ap, 0, this, 0);
	ap = a;
	if (ap)
		connect(ap, SIGNAL(changed()), this, SIGNAL(changed()));
}

void
NAV::setNavaid(Navaid *n)
{
	if (nav)
		disconnect(nav, 0, this, 0);
	nav = n;
	if (nav)
		connect(nav, SIGNAL(changed()), this, SIGNAL(changed()));
}

int
NAV::radial()
{
	if (!isTuned())
		return 0;

	double r = M_PI_2 - atan2(ap->y() - nav->y(), ap->x() - nav->x());
	r *= 180. / M_PI;
	r = fmod(r + 360., 360.);

	return (int)r;
}

double
NAV::dme()
{
	if (!isTuned())
		return -1;

	double dx = ap->x() - nav->x();
	double dy = ap->y() - nav->y();
	return sqrt(dx*dx + dy*dy);
}

int
NAV::relativeBearing()
{
	if (!isTuned())
		return 0;

	return (radial() + 180 + ap->heading()) % 360;
}

int
NAV::CDIInfo(double &defl, int obs)
{
	int flag;

	if (!isTuned()) {
		defl = 0.;
		return Off;
	}

	int rad = radial();
	double offcourse = fmod(-obs - rad + 720., 360.);

	if (offcourse > 92. && offcourse < 268.) {
		flag = From;
	} else if (offcourse < 88. || offcourse > 272.) {
		flag = To;
	} else {
		flag = Off;
	}

	if (offcourse > 90. && offcourse < 270.) {
		offcourse = 180. - offcourse;
	} else if (offcourse <= 90.) {
		;
	} else if (offcourse >= 270.) {
		offcourse -= 360.;
	}

	if (offcourse < -10. || offcourse > 10.)
		defl = copysign(10., offcourse);
	else
		defl = offcourse;

	if (dme() < 5)
		flag = Off;

	return flag;
}
