/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: Navaid.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include "Navaid.h"

Navaid::Navaid(QCanvas *canvas, const char *name, CanvasRSprite *it)
{
	canv_img = it;
	canv_label = new QCanvasText(name, canvas);
	connect(canv_img, SIGNAL(dragged()), this, SLOT(canvImgChanged()));
	canvImgChanged();
	show();
}

QPoint
Navaid::labelOffset()
{
	QRect r = canv_img->boundingRect();
	return QPoint(r.width(), -r.height());
}

void
Navaid::canvImgChanged()
{
	QPoint offs = labelOffset();
	canv_label->move(canv_img->x() + offs.x(),
			 canv_img->y() + offs.y());
	emit changed();
}

void
Navaid::move(int x, int y)
{
	canv_img->move(x, y);
	canvImgChanged();
}

bool
Navaid::isVisible()
{
	// return canv_img->isVisible();
	return canv_img->visible();
}

void
Navaid::setVisible(bool t)
{
	if (t != isVisible()) {
		canv_img->setVisible(t);
		canv_label->setVisible(t);
		canv_img->canvas()->update();
		emit changed();
	}
}

void
Navaid::show()
{
	setVisible(TRUE);
}

void
Navaid::hide()
{
	setVisible(FALSE);
}
