/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: RMI.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include "RMI.h"
#include <math.h>
#include <qpixmap.h>
#include "Embed.h"

RMI::RMI(QWidget *parent, const char *name, WFlags flags)
	:ADF(parent, name, flags)
{
	is_slaved = TRUE;
	is_fixed = TRUE;
	nav_radio2 = NULL;
}

const QPixmap &
RMI::overlayPixmap()
{
	static const QPixmap *p;

	if (!p)
		p = &embed_QPixmap("images/RMIOverlay.png");
	return *p;
}

void
RMI::render()
{
	ADF::render();
	int b = nav_radio2 ? nav_radio2->relativeBearing() : 0;
	drawBearingArrow2(b);
}

void
RMI::drawBearingArrow2(int b)
{
	QPainter p(&pm);
	p.setPen(QPen(Qt::green, 2,
			Qt::SolidLine,
			Qt::SquareCap,
			Qt::BevelJoin));
	const int size = pm.height() / 16 * 6;
	QPoint center(pm.width() / 2, pm.height() / 2);
	double angle = -(b / 180. * M_PI) + M_PI/2;
	double rangle = angle + M_PI_2;

	double ca = cos(angle), sa = sin(angle);
	double rca = cos(rangle), rsa = sin(rangle);

	QPoint head(center.x() + ca * size,
		    center.y() + sa * size);
	QPoint subhead(center.x() + ca * size * 5 / 7,
		    center.y() + sa * size * 5 / 7);
	QPoint tail(center.x() - ca * size,
		    center.y() - sa * size);
	QPoint subtail(center.x() - ca * size * 2 / 3,
		       center.y() - sa * size * 2 / 3);
	QPoint subsubtail(center.x() - ca * size * 16 / 30,
		          center.y() - sa * size * 16 / 30);
	QPoint roffs(rca * size / 10, rsa * size / 10);
	QPoint loffs = -roffs;

	QPointArray arrow(10);
	arrow[0] = tail;
	arrow[1] = subtail;
	arrow[2] = subsubtail + roffs;
	arrow[3] = subhead + roffs;
	arrow[4] = subhead + roffs * 2;
	arrow[5] = head;
	arrow[6] = subhead + loffs * 2;
	arrow[7] = subhead + loffs;
	arrow[8] = subsubtail + loffs;
	arrow[9] = subtail;

	p.drawPolyline(arrow);
}

// set 2 NAV radios by calling this twice.  oldest one falls off
void
RMI::setNAV(NAV *n)
{
	// Trick superclass into doing work
	ADF::setNAV(n);

	// Swap them to get LIFO behavior
	NAV *t = nav_radio2;
	nav_radio2 = nav_radio;
	nav_radio = t;
}
