/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: Rotate.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include <math.h>
#include <stdlib.h>
#include <qimage.h>
#include "LineIterator.h"

#undef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#undef abs
#define abs(a) ((a) > (0) ? (a) : -(a))

QImage
RotateQImage(const QImage &in, double angle)
{
	int width = in.width(), height = in.height();
	int cx = width / 2, cy = height / 2;
	double r = max(cx, cy) * M_SQRT2;
	angle = angle / 180. * M_PI + M_PI/2.;
	int ulx = (int)rint(cos(angle + M_PI/4.) * r) + cx;
	int uly = (int)rint(-sin(angle + M_PI/4.) * r) + cy;
	int llx = (int)rint(cos(angle + M_PI/4. + M_PI/2.) * r) + cx;
	int lly = (int)rint(-sin(angle + M_PI/4. + M_PI/2.) * r) + cy;
	int nheight = max(abs(llx - ulx), abs(lly - uly)) + 1;
	double cosd = cos(angle - M_PI/2.);
	double sind = -sin(angle - M_PI/2.);
	int dx, dy;
	if (fabs(cosd) > fabs(sind)) {
		dx = (int)rint(copysign((double)nheight - 1, cosd));
		dy = (int)rint(sind * (dx / cosd));
	} else {
		dy = (int)rint(copysign((double)nheight - 1, sind));
		dx = (int)rint((cosd * (dy / sind)));
	}
	int nwidth = max(abs(dx), abs(dy)) + 1;

	QImage image(nwidth, nheight, in.depth(), in.numColors());
	for (int i = in.numColors() - 1; i >= 0; --i) {
		image.setColor(i, in.color(i));
	}

	uchar **inp = in.jumpTable();
	uchar **outp = image.jumpTable();

	LineIterator v(ulx, uly, llx, lly);
	int x, y = 0;
	while (v.next()) {
		LineIterator h(v.x, v.y, v.x + dx, v.y + dy);
		x = 0;
		while (h.next()) {
			// XXX in.valid()
			if (h.x > 0 && h.x < width &&
			    h.y > 0 && h.y < height)
				outp[y][x] = inp[h.y][h.x];
			else
				outp[y][x] = 0;
			x++;
		}
		y++;
	}
	return image;
}
