/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: SimCanvas.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include "SimCanvas.h"
#include <qpixmap.h>
#include <qpainter.h>

const int DragCanvasItemI::RTTI =    814178733;
const int DragCanvasItemI::MaxRTTI = 814178733 + 100;

void
SimCanvasView::resizeEvent(QResizeEvent *e)
{
	QCanvasView::resizeEvent(e);
	canvas()->resize(visibleWidth(), visibleHeight());
}
void
SimCanvasView::contentsMouseMoveEvent(QMouseEvent *e)
{
	if (moving) {
		moving->mouseMoveEvent(e);
	}
}

// ZZZ ohh this is ugly, but I need dynamic_cast<> and it's not in embed
class ZZZ : public DragCanvasItemI, public QCanvasSprite
{
	public: ZZZ() : QCanvasSprite(0,0) {;}
	QCanvasItem *canvasItem() const { return 0;}
};
static DragCanvasItemI *
cast_DragCanvasItemI(QCanvasItem *i)
{
	ZZZ z;
	char *p1 = (char *)((DragCanvasItemI *)&z);
	char *p2 = (char *)((QCanvasSprite *)&z);
	char *res = (char *)i;

	return (DragCanvasItemI *) (res - p2 + p1);
}

void
SimCanvasView::contentsMousePressEvent(QMouseEvent *e)
{
	QCanvasItemList l = canvas()->collisions(e->pos());
	for (QCanvasItemList::Iterator it = l.begin(); it != l.end(); ++it) {
		if ((*it)->rtti() >= DragCanvasItemI::RTTI &&
		    (*it)->rtti() <= DragCanvasItemI::MaxRTTI) {
			// moving = dynamic_cast<DragCanvasItemI *> (*it);
			moving = cast_DragCanvasItemI(*it);
			moving->mousePressEvent(e);
			return;
		}
	}
	moving = NULL;
}

SimCanvas::SimCanvas()
{
	const int tilesize = 50;
	QPixmap grid(tilesize, tilesize);
	grid.fill(Qt::white);
	QPainter p(&grid);
	p.setPen(QColor(200, 235, 200));
	p.drawLine(0, tilesize/2, tilesize-1, tilesize/2);
	p.drawLine(tilesize/2, 0, tilesize/2, tilesize-1);
	for (int i = 5; i < tilesize; i += 10) {
		p.drawLine(tilesize/2-2, i, tilesize/2+2, i);
		p.drawLine(i, tilesize/2-2, i, tilesize/2+2);
	}

	setDoubleBuffering(TRUE);
	setBackgroundPixmap(grid);
}

int
DragCanvasItemI::bodypartAction(QPoint const &p) const
{
	return Move;
}

void
DragCanvasItemI::mousePressEvent(QMouseEvent *e)
{
	QCanvasItem *ci = canvasItem();

	moveOrigin = e->pos() - QPoint(ci->x(), ci->y());
	moveAction = bodypartAction(unRotate(moveOrigin));
	rotateOrigin = ang - atan2(moveOrigin.y(), moveOrigin.x());
}

void
DragCanvasItemI::mouseMoveEvent(QMouseEvent *e)
{
	QCanvasItem *ci = canvasItem();
	QPoint p = e->pos() - QPoint(ci->x(), ci->y());

	if (!ci)
		return;

	switch(moveAction) {
	case Move: 
		if (ci->canvas()->onCanvas(e->pos())) {
			QPoint relative = p - moveOrigin;
			ci->moveBy(relative.x(), relative.y());
			moveOrigin = e->pos() - QPoint(ci->x(), ci->y());
		}
		break;
	case Rotate: {
		double angle = atan2(p.y(), p.x());
		setAngle(rotateOrigin + angle);
		break;
	}
	case Tow:
		break;
	}
	emit dragged();
	ci->canvas()->update();
}

QPoint
DragCanvasItemI::unRotate(QPoint const &p) const
{
	double c = cos(-ang);
	double s = sin(-ang);

	return QPoint(c * p.x() - s * p.y(),
			s * p.x() + c * p.y());
}

