/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: SimCanvas.h,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#ifndef __SIMCANVAS_H
#define __SIMCANVAS_H

#include <qcanvas.h>
#include <math.h>

class DragCanvasItemI : public QObject
{
	Q_OBJECT
public:
	static const int RTTI;
	static const int MaxRTTI;

	virtual void setAngle(double a) { ang = fmod(a + 8*M_PI, 2*M_PI); }
	double angle() const { return ang; }

protected:
	QPoint				moveOrigin;
	enum { Rotate, Move, Tow };
	int				moveAction;
	double				rotateOrigin;

	friend class SimCanvas;
	friend class SimCanvasView;

	virtual void mouseMoveEvent(QMouseEvent *e);
	virtual void mousePressEvent(QMouseEvent *e);
	virtual int bodypartAction(QPoint const &p) const;
	QPoint unRotate(QPoint const &p) const;

	virtual QCanvasItem *canvasItem() const = 0;

signals:
	void dragged();

private:
	double	ang;
};

class SimCanvasView : public QCanvasView
{
	Q_OBJECT

public:
	SimCanvasView(QCanvas *canv, QWidget *parent)
		: QCanvasView(canv, parent) { }

private:
	DragCanvasItemI		*moving;

protected:
	virtual void resizeEvent(QResizeEvent *e);
	virtual void contentsMouseMoveEvent(QMouseEvent *e);
	virtual void contentsMousePressEvent(QMouseEvent *e);
};

class SimCanvas : public QCanvas {
public:
	SimCanvas();
};

#endif // __SIMCANVAS_H
