/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: SimCanvasAirplane.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include <math.h>
#include "SimCanvasAirplane.h"

const int SimCanvasAirplane::RTTI = 814178734;

static const int mypoints[] = {
	0,-12,
	1,-12,
	3,-10,
	3, -4,
	23,-4,
	23, 3,
	11, 6,
	3,  6,
	1,  20,
	10, 23,
	10, 27,
};
#define NPOINTS (sizeof(mypoints)/sizeof(int)/2)

SimCanvasAirplane::SimCanvasAirplane(QCanvas *canvas, int heading = 0)
	: CanvasRPolygon(canvas, -heading/180.*M_PI)
{
	setBrush(QColor(0, 0, 225));
	QPointArray a(NPOINTS, mypoints);
	a.resize(a.size() * 2);
	for (int i = 0, j = a.size() - 1; i < j; ++i, --j)
		a[j] = QPoint(-a[i].x(), a[i].y());
	setBasePoints(a);
	setZ(3000);
}

int
SimCanvasAirplane::bodypartHit(QPoint const &p) const
{
	if (p.x() < -6 || p.x() > 6)
		return Wing;
	if (p.y() < -4)
		return Nose;
	if (p.y() >= 20)
		return Tail;
	return Fuselage;
}

int
SimCanvasAirplane::bodypartAction(QPoint const &p) const
{
	switch (bodypartHit(p)) {
	case Wing:
	case Tail:
		return Rotate;
	case Nose:
	case Fuselage:
		return Move;
	}
	return Move; // XXX
}
