/*
 * Copyright (C) 2002 Ben J Jackson <ben@ben.com>.  All rights reserved.
 *
 * This file is part of Instrument Trainer for Sharp Zaurus, written for
 * the Sharp Developer Challenge:  http://www.zaurus.com/dev/challenge/
 *
 * See http://www.ben.com/ for the latest version of this software.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//static char rcsid[] = "$Id: top.cpp,v 1.1 2002/10/06 04:17:00 bjj Rel $";

#include "top.h"
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qcanvas.h>
#include <qabstractlayout.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qwidgetstack.h>
#include <qdialog.h>

#include "SimCanvas.h"
#include "Airplane.h"
#include "VOR.h"
#include "NDB.h"

#include "Embed.h"

void
Top::showHide(int id)
{
	Navaid *n = navaids[id];

	bool b = ! navaidMenu->isItemChecked(id);
	navaidMenu->setItemChecked(id, b);
	n->setVisible(b);
}

void
Top::rightStack(int id)
{
	QWidget *w = ap->instruments[id];
	int old = rcur;

	if (old == id)
		return;
	if (w->parentWidget() != rstack) {
		w->reparent(rstack, w->pos());
		rstack->addWidget(w, id);
	}
	rstack->raiseWidget(w);
	rcur = id;
	if (lcur == id) {
		lcur = -1;
		leftStack(old);
	}
}


void
Top::leftStack(int id)
{
	QWidget *w = ap->instruments[id];
	int old = lcur;

	if (old == id)
		return;
	if (w->parentWidget() != lstack) {
		w->reparent(lstack, w->pos());
		lstack->addWidget(w, id);
	}
	lstack->raiseWidget(w);
	lcur = id;
	if (rcur == id) {
		rcur = -1;
		rightStack(old);
	}
}


Top::Top(QWidget *parent = 0, const char *name = 0, WFlags f = WType_TopLevel)
	: QMainWindow(parent, name, f)
{
	setCaption("Instrument Trainer");

#if 0
	fileMenu = new QPopupMenu;
	fileMenu->insertItem("E&xit", this, SLOT(Exit()));
	menuBar()->insertItem("&File", fileMenu);
#endif

	QPopupMenu *lsel = new QPopupMenu;
	// lsel->insertItem("&Tune", this, SLOT(Popup()), 0, 99);
	// lsel->insertSeparator();
	lsel->insertItem("&DG", this, SLOT(leftStack(int)), 0, Airplane::id_DG);
	lsel->insertItem("VOR &1", this, SLOT(leftStack(int)), 0, Airplane::id_VOR1);
	lsel->insertItem("VOR &2", this, SLOT(leftStack(int)), 0, Airplane::id_VOR2);
	lsel->insertItem("&ADF", this, SLOT(leftStack(int)), 0, Airplane::id_ADF);
	lsel->insertItem("&RMI", this, SLOT(leftStack(int)), 0, Airplane::id_RMI);
	menuBar()->insertItem(embed_QPixmap("images/menuLeftInst.png"), lsel);

	QPopupMenu *rsel = new QPopupMenu;
	// rsel->insertItem("&Tune", this, SLOT(Popup()), 0, 99);
	// rsel->insertSeparator();
	rsel->insertItem("&DG", this, SLOT(rightStack(int)), 0, Airplane::id_DG);
	rsel->insertItem("VOR &1", this, SLOT(rightStack(int)), 0, Airplane::id_VOR1);
	rsel->insertItem("VOR &2", this, SLOT(rightStack(int)), 0, Airplane::id_VOR2);
	rsel->insertItem("&ADF", this, SLOT(rightStack(int)), 0, Airplane::id_ADF);
	rsel->insertItem("&RMI", this, SLOT(rightStack(int)), 0, Airplane::id_RMI);
	menuBar()->insertItem(embed_QPixmap("images/menuRightInst.png"), rsel);

	navaidMenu = new QPopupMenu;
	navaidMenu->insertItem("Show VOR &1", this, SLOT(showHide(int)), 0, 0);
	navaidMenu->insertItem("Show VOR &2", this, SLOT(showHide(int)), 0, 1);
	navaidMenu->insertItem("Show NDB", this, SLOT(showHide(int)), 0, 2);
	for (int i = 0; i < 3; ++i)
		navaidMenu->setItemChecked(i, TRUE);
	menuBar()->insertItem(embed_QPixmap("images/NavaidMenu.png"), navaidMenu);

	// should work, but Qt authors on crack?
	// menuBar()->insertSeparator();
	menuBar()->insertItem(new QWidget(menuBar()));
	menuBar()->insertItem(embed_QPixmap("images/menuQuit.png"), this, SLOT(Exit()));
	
	QVBox *vb = new QVBox(this);
	QHBox *hb = new QHBox(vb);

	lstack = new QWidgetStack(hb);
	rstack = new QWidgetStack(hb);
	rcur = lcur = -1;

	int size = 120;
	lstack->setFixedSize(size, size);
	rstack->setFixedSize(size, size);

	SimCanvas *c = new SimCanvas();
	cv = new SimCanvasView(c, vb);
	ap = new Airplane(c);

	navaids[0] = new VOR(c, "UBG");
	navaids[1] = new VOR(c, "BTG");
	navaids[2] = new NDB(c, "MM");

	// tuning
	ap->navs[0]->setNavaid(navaids[0]);
	ap->navs[1]->setNavaid(navaids[1]);
	ap->navs[2]->setNavaid(navaids[2]);

	ap->move(120, 70);
	navaids[0]->move(70, 100);
	navaids[1]->move(170, 50);
	navaids[2]->move(20, 20);

	leftStack(Airplane::id_DG);
	rightStack(Airplane::id_VOR1);

	setCentralWidget(vb);
}

void
Top::Exit()
{
	exit(0);
}

void
Top::Popup()
{
	QDialog *bob = new QDialog();
	bob->exec();
	delete bob;
}
